﻿//v0.2
using System;
using KSP.IO;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class subassemblyLoader : MonoBehaviour
{
    public static GameObject GameObjectInstance;
    private static PluginConfiguration config = PluginConfiguration.CreateForType<subassemblyLoader>();
    private static string root = KSPUtil.ApplicationRootPath.Replace("\\", "/");
    private static string subassemblyFolder = "PluginData/subassemblyLoader/subassemblies";
    
    private string subassemblyFilename = "Untitled";
    private string subassemblyType = "";
    private string subassemblyFolderName = root + subassemblyFolder;
    private bool showFilenamePrompt;
    private bool isAttachable = true;
    private FileBrowser fileBrowser;

    private GUIStyle buttonStyle = new GUIStyle();

    // Load static button images
    private WWW img1 = new WWW("file://" + root + "PluginData/subassemblyLoader/icons/buttonDisabled.png");
    private WWW img2 = new WWW("file://" + root + "PluginData/subassemblyLoader/icons/buttonInactive.png");
    private WWW img3 = new WWW("file://" + root + "PluginData/subassemblyLoader/icons/buttonHover.png");
    private WWW img4 = new WWW("file://" + root + "PluginData/subassemblyLoader/icons/file.png");
    private WWW img5 = new WWW("file://" + root + "PluginData/subassemblyLoader/icons/folder.png");

    private Texture2D buttonDisabled = new Texture2D(0, 0);
    private Texture2D buttonInactive = new Texture2D(0, 0);
    private Texture2D buttonHover = new Texture2D(0, 0);
    private Texture2D fileIcon = new Texture2D(0, 0);
    private Texture2D folderIcon = new Texture2D(0, 0);
    private Texture2D button;

    public void Awake()
    {
        DontDestroyOnLoad(this);

        // Save static button images to textures
        img1.LoadImageIntoTexture(buttonDisabled);
        img2.LoadImageIntoTexture(buttonInactive);
        img3.LoadImageIntoTexture(buttonHover);
        img4.LoadImageIntoTexture(fileIcon);
        img5.LoadImageIntoTexture(folderIcon);
    }

    public void saveSubassembly(string filename)
    {
        //Close Filename Prompt
        showFilenamePrompt = false;
        EditorLogic.fetch.Unlock();

        // Create new construct
        ShipConstruct subassemblyConstruct = new ShipConstruct(1);
        subassemblyConstruct.shipName = filename;

        // Add selected part
        Part selected = EditorLogic.fetch.PartSelected;
        subassemblyConstruct.Add(selected);

        // Add selected part's children
        Part[] children = selected.FindChildParts<Part>(true);
        foreach (Part part in children) subassemblyConstruct.Add(part);

        // Save construct
        Debug.Log("../../../../" + subassemblyFolder + '/' + subassemblyType + '/' + filename);
        ShipConstruction.SaveShip(subassemblyConstruct, "../../../../" + subassemblyFolder + '/' + subassemblyType + '/' + filename);
    }

    public void loadSubassembly(string filename)
    {
        EditorLogic editor = EditorLogic.fetch;

        //Close Subassembly Browser
        fileBrowser = null;
        editor.Unlock(); 
        
        if (filename == null) return;

        // Get current root, load subassembly construct, reset root
        Part shipRoot = EditorLogic.startPod;
        ShipConstruct subassemblyConstruct = ShipConstruction.LoadShip(filename);
        EditorLogic.startPod = shipRoot;
        
        // Select new construct
        if( editor.PartSelected ) EditorLogic.DeletePart(editor.PartSelected);
        editor.PartSelected = subassemblyConstruct.Parts[0].localRoot;

        // Check new construct is attachable
        checkAttachable(editor.PartSelected);
    }

    // Check if part is attachable
    private bool checkAttachable(Part part)
    {
        isAttachable = part.attachRules.srfAttach;
        if (!isAttachable)
            foreach (AttachNode node in part.attachNodes)
                if (!node.attachedPart) isAttachable = true;
        return isAttachable;
    }





// The rest of this entire file a giant clusterfuck of kludges upon hacks in order to get a working file browser despite KSP.IOs total lack of directory methods.


    private void OnGUI()
    {
        EditorLogic editor = EditorLogic.fetch;
        if (!editor) return;

        GUI.skin = HighLogic.Skin;

        // Lock editor if we have a GUI window open
        if ((fileBrowser != null || showFilenamePrompt || !isAttachable) && !EditorLogic.softLock) editor.Lock(true, true, true);

        // Bad subassembly warning
        if (!isAttachable)
            GUILayout.Window(3, new Rect(Screen.width / 2, 100, 350, 10), badSubassemblyGUI, "Warning: Bad subassembly!");
            

        // Draw top menu button
        if (GUI.Button(new Rect(Screen.width - 300, 1, 48, 36), new GUIContent(button, "Subassembly"), buttonStyle) && !EditorLogic.softLock)
            if (EditorLogic.SelectedPart)
            {
                if (checkAttachable(EditorLogic.SelectedPart))
                    showFilenamePrompt = true;
            }
            else
                createFileBrowser("Select a subassembly to load", false, loadSubassembly);

        // Figure out what texture to use for the rollover
        if( Event.current.type == EventType.Repaint ) 
        {
            button = buttonInactive;
            if (EditorLogic.softLock)
                button = buttonDisabled;
            else if ( GUI.tooltip == "Subassembly")
                button = buttonHover;
        }

        // Show FileBrowser
        if (fileBrowser != null)
        {
            // Set skin options
            GUI.skin = editor.shipBrowserSkin;
            GUIStyle list = GUI.skin.FindStyle("List Item");
                list.normal.textColor = XKCDColors.DarkYellow; 
                list.contentOffset = new Vector2(1, 0);
                list.fontSize = 20;

            string current = fileBrowser.CurrentDirectory.Replace('\\', '/');
            subassemblyType = "";
            if( current != root + subassemblyFolder) subassemblyType = current.Substring((root + subassemblyFolder).Length + 1);

            // Draw file browser
            fileBrowser.OnGUI();

            // Draw "Up" button (if applicable)
            if (current != root + subassemblyFolder)
                if (GUI.Button(new Rect(Screen.width / 2 + 190, 562, 40, 30), "/.."))
                    fileBrowser.CurrentDirectory = current.Substring(0, current.LastIndexOf('/'));

            // Reset Skin
            list.normal.textColor = new Color(.739f, 0.739f, 0.739f);
            list.contentOffset = new Vector2(1, 42.4f);
            list.fontSize = 10;
        }

        // Save Subassembly Prompt
        if (showFilenamePrompt)
            if (fileBrowser == null)
                GUILayout.Window(1, new Rect(Screen.width / 2, 100, 350, 128), saveGUI, "Save Subassembly");
            else if (fileBrowser.CurrentDirectory != root + subassemblyFolder)
            {
                subassemblyFolderName = fileBrowser.CurrentDirectory;
                fileBrowser = null;
            }
    }

    // Subassembly filename Prompt
    private void saveGUI(int windowID)
    {
        if (subassemblyFolderName != root + subassemblyFolder)
            subassemblyType = subassemblyFolderName.Substring((root + subassemblyFolder).Length + 1);

        // Name field
        GUILayout.BeginHorizontal();
        GUILayout.Label("Name:",GUILayout.Width(75));
        subassemblyFilename = GUILayout.TextField(subassemblyFilename, 128);
        GUILayout.EndHorizontal();

        // Category button
        GUILayout.BeginHorizontal(); 
        GUILayout.Label("Category:", GUILayout.Width(75));
        if (GUILayout.Button(subassemblyType == "" ? "Miscellaneous" : subassemblyType))
            createFileBrowser("Select subassembly category", true, (string filename) => { subassemblyFolderName = fileBrowser.CurrentDirectory; fileBrowser = null; });
        GUILayout.EndHorizontal();

        // Ok / Cancel buttons
        GUILayout.BeginHorizontal();
        GUILayout.FlexibleSpace();

        if (GUILayout.Button("Save"))
            saveSubassembly(subassemblyFilename);

        if (GUILayout.Button("Cancel"))
        {
            showFilenamePrompt = false;
            EditorLogic.fetch.Unlock();
        }
        GUILayout.EndHorizontal();
    }

    // Can't attach warning
    private void badSubassemblyGUI(int windowID)
    {
        GUILayout.BeginVertical();
        GUILayout.Label("Root part of selected subassembly has no free nodes and doesn't allow surface attachment, so cannot be attached to anything.");
        if (GUILayout.Button("Cancel"))
        {
            EditorLogic.fetch.Unlock();
            EditorLogic.DeletePart(EditorLogic.SelectedPart);
            isAttachable = true;
            showFilenamePrompt = false;
            fileBrowser = null;
        }
        GUILayout.EndVertical();
    }

    // Create file browser
    private void createFileBrowser(string title,bool isDir, FileBrowser.FinishedCallback callback)
    {
        fileBrowser = new FileBrowser(new Rect(Screen.width / 2, 100, 350, 500), title, callback, true);
        fileBrowser.BrowserType = isDir ? FileBrowserType.Directory : FileBrowserType.File;
        fileBrowser.CurrentDirectory = root + subassemblyFolder;
        fileBrowser.FileImage = fileIcon;
        fileBrowser.DirectoryImage = folderIcon;
        fileBrowser.disallowDirectoryChange = true;
        if( !isDir ) fileBrowser.SelectionPattern = "*.craft";
    }
}

public class subassemblyLoaderInit : KSP.Testing.UnitTest
{
    public subassemblyLoaderInit()
    {
        var gameobject = new GameObject("subassemblyLoader", typeof(subassemblyLoader));
        UnityEngine.Object.DontDestroyOnLoad(gameobject);
    }
}